import 'package:flutter/services.dart';

import '../models/data.dart';
import '../models/data_result.dart';
import '../models/field.dart';
import '../models/location.dart';
import '../models/order.dart';
import '../platform_interface/client.dart';
import '../platform_interface/data_query.dart';
import '../utils/mappings.dart';
import 'method_channel_client.dart';

class MethodChannelDataQuery extends DataQueryPlatform with MethodChannelQuery {
  late MethodChannel _channel;

  String? _objectId;
  String? _userAccessToken;

  bool? _withFavorite;
  bool? _withFavoriteSummary;

  MethodChannelDataQuery(MethodChannel channel, String objectId,
      {String? userAccessToken}) {
    _channel = channel;
    _objectId = objectId;
    _userAccessToken = userAccessToken;
  }

  @override
  void withFavorite({bool? byMe}) {
    _withFavorite = true;
    if (byMe != null) {
      equalTo('sys_favorite:is_favorite', exportBool(byMe)!);
    }
  }

  @override
  void withFavoriteSummary() {
    _withFavoriteSummary = true;
  }

  @override
  void orderByFavoriteUpdatedAt({Direction direction = Direction.asc}) {
    order.add(
        Order(direction: direction, columnName: 'sys_favorite:update_dte'));
  }

  @override
  void orderByFavoriteCount({Direction direction = Direction.asc}) {
    order.add(Order(
        direction: direction, columnName: 'sys_favorite_sum:favorite_count'));
  }

  Map<String, dynamic> _exportExtensionAttribute() {
    return <String, dynamic>{
      'userAccessToken': _userAccessToken,
      'showFavorite': _withFavorite,
      'showFavoriteSummary': _withFavoriteSummary
    };
  }

  @override
  Future<DataResult> find() async {
    if (limit != null) {
      final data = await _channel
          .invokeMapMethod<String, dynamic>('getPaginateDataList', {
        'objectId': _objectId,
        'limit': limit,
        'offset': offset ?? 0,
        'searchConditions': exportSearchConditions(),
        'sortConditions': exportSortConditions(),
        'extensionAttribute': _exportExtensionAttribute(),
      });
      return DataResult.fromMap(data!);
    }
    final data = await _channel.invokeListMethod<Map>('getDataList', {
      'objectId': _objectId,
      'searchConditions': exportSearchConditions(),
      'sortConditions': exportSortConditions(),
      'extensionAttribute': _exportExtensionAttribute()
    });
    return DataResult(
        resultCnt: data!.length,
        data: data.map(importMap).map((data) => Data.fromMap(data!)).toList());
  }

  @override
  Future<DataResult> findWithRelation({int? treeCount}) async {
    if (limit != null) {
      final data = await _channel.invokeMapMethod<String, dynamic>(
          'getPaginateDataListWithRelationObjects', {
        'objectId': _objectId,
        'limit': limit,
        'offset': offset ?? 0,
        'treeCount': treeCount,
        'searchConditions': exportSearchConditions(),
        'sortConditions': exportSortConditions(),
      });
      return DataResult.fromMap(data!);
    }
    final data =
        await _channel.invokeListMethod<Map>('getDataListWithRelationObjects', {
      'objectId': _objectId,
      'treeCount': treeCount,
      'searchConditions': exportSearchConditions(),
      'sortConditions': exportSortConditions()
    });
    return DataResult(
        resultCnt: data!.length,
        data: data.map(importMap).map((data) => Data.fromMap(data!)).toList());
  }

  @override
  Future<DataResult> findByLocation(
      {RKZLocation? location, String? spotFieldName}) async {
    if (limit != null) {
      final data = await _channel
          .invokeMapMethod<String, dynamic>('getPaginateDataListWithLocation', {
        'objectId': _objectId,
        'limit': limit,
        'offset': offset ?? 0,
        'location': location?.toMap(),
        'spotFieldName': spotFieldName,
        'searchConditions': exportSearchConditions(),
        'sortConditions': exportSortConditions(),
      });
      return DataResult.fromMap(data!);
    }
    final data =
        await _channel.invokeListMethod<Map>('getDataListWithLocation', {
      'objectId': _objectId,
      'location': location?.toMap(),
      'spotFieldName': spotFieldName,
      'searchConditions': exportSearchConditions(),
      'sortConditions': exportSortConditions()
    });
    return DataResult(
        resultCnt: data!.length,
        data: data.map(importMap).map((data) => Data.fromMap(data!)).toList());
  }

  @override
  Future<int> delete() async {
    return (await _channel.invokeMethod<int>('deleteData', {
      'objectId': _objectId,
      'searchConditions': exportSearchConditions()
    }))!;
  }
}

mixin MethodChannelDataClient on RKZClientPlatform
    implements MethodChannelClientInterface {
  @override
  MethodChannelDataQuery dataQuery(String objectId, {String? userAccessToken}) {
    return MethodChannelDataQuery(channel, objectId,
        userAccessToken: userAccessToken);
  }

  @override
  Future<Data> getData(String objectId, String code) async {
    final data = await channel.invokeMapMethod<String, dynamic>(
        'getData', {'objectId': objectId, 'code': code});
    return Data.fromMap(data!);
  }

  @override
  Future<Data> getDataWithRelation(String objectId, String code,
      {int? treeCount}) async {
    final data = await channel
        .invokeMapMethod<String, dynamic>('getDataWithRelationObjects', {
      'objectId': objectId,
      'code': code,
      'treeCount': treeCount,
    });
    return Data.fromMap(data!);
  }

  @override
  Future<Data> getDataByLocation(String objectId, String code,
      {RKZLocation? location, String? spotFieldName}) async {
    final data =
        await channel.invokeMapMethod<String, dynamic>('getDataWithLocation', {
      'objectId': objectId,
      'code': code,
      'location': location?.toMap(),
    });
    return Data.fromMap(data!);
  }

  @override
  Future<void> addData(Data data) {
    return channel.invokeMethod('addData', {'data': data.toMap()});
  }

  @override
  Future<void> editData(Data data) {
    return channel.invokeMethod('editData', {'data': data.toMap()});
  }

  @override
  Future<List<Field>> getDataFields(String objectId,
      {bool? visibleOnly = true}) async {
    final data = await channel.invokeListMethod<Map>(
        'getFieldDataList', {'objectId': objectId, 'visibleOnly': visibleOnly});
    return data!.map(importMap).map((data) => Field.fromMap(data!)).toList();
  }

  @override
  Future<Data> getDataByQRCode(String qrCode) async {
    final data = await channel.invokeMapMethod<String, dynamic>(
        'getDataFromQRCode', {'qrCode': qrCode});
    return Data.fromMap(data!);
  }

  @override
  Future<void> addDataToFavorite(
      String userAccessToken, String objectId, String code) {
    return channel.invokeMethod('addDataToFavorite', {
      'userAccessToken': userAccessToken,
      'objectId': objectId,
      'code': code,
    });
  }

  @override
  Future<void> deleteDataFromFavorite(
      String userAccessToken, String objectId, String code) {
    return channel.invokeMethod('deleteDataFromFavorite', {
      'userAccessToken': userAccessToken,
      'objectId': objectId,
      'code': code,
    });
  }
}
