import 'package:flutter/services.dart';

import '../models/news.dart';
import '../models/news_result.dart';
import '../platform_interface/client.dart';
import '../platform_interface/news_query.dart';
import '../types.dart';
import '../utils/mappings.dart';
import 'method_channel_client.dart';

class MethodChannelNewsQuery extends NewsQueryPlatform with MethodChannelQuery {
  late MethodChannel _channel;

  String? _userAccessToken;

  bool? _withFavorite;
  bool? _withFavoriteSummary;
  bool? _segmentMatchOnly;

  MethodChannelNewsQuery(MethodChannel channel, {String? userAccessToken}) {
    _channel = channel;
    _userAccessToken = userAccessToken;
  }

  @override
  void alreadyRead() {
    equalTo('read_history_kbn', ReadHistoryType.alreadyRead);
  }

  @override
  void unread() {
    equalTo('read_history_kbn', ReadHistoryType.unread);
  }

  @override
  void withFavorite() {
    _withFavorite = true;
  }

  @override
  void withFavoriteSummary() {
    _withFavoriteSummary = true;
  }

  @override
  void segmentMatchOnly() {
    _segmentMatchOnly = true;
  }

  @override
  Future<NewsResult> all() async {
    if (_userAccessToken != null) {
      if (limit != null && offset != null) {
        final data = await _channel
            .invokeMapMethod<String, dynamic>('getPaginateSegmentNewsList', {
          'limit': limit,
          'offset': offset,
          'userAccessToken': _userAccessToken,
          'segmentMatchOnly': _segmentMatchOnly,
          'searchConditions': exportSearchConditions(),
          'sortConditions': exportSortConditions(),
          'extensionAttribute': _exportExtensionAttribute()
        });
        return NewsResult.fromMap(data!);
      }
      final data = await _channel.invokeListMethod<Map>('getSegmentNewsList', {
        'limit': limit,
        'userAccessToken': _userAccessToken,
        'segmentMatchOnly': _segmentMatchOnly,
        'searchConditions': exportSearchConditions(),
        'sortConditions': exportSortConditions(),
        'extensionAttribute': _exportExtensionAttribute()
      });
      return NewsResult(
          resultCnt: data!.length,
          data:
              data.map(importMap).map((data) => News.fromMap(data!)).toList());
    }
    if (limit != null && offset != null) {
      final data = await _channel
          .invokeMapMethod<String, dynamic>('getPaginateNewsList', {
        'limit': limit,
        'offset': offset,
        'searchConditions': exportSearchConditions(),
        'sortConditions': exportSortConditions(),
        'extensionAttribute': _exportExtensionAttribute()
      });
      return NewsResult.fromMap(data!);
    }
    final data = await _channel.invokeListMethod<Map>('getNewsList', {
      'limit': limit,
      'searchConditions': exportSearchConditions(),
      'sortConditions': exportSortConditions(),
      'extensionAttribute': _exportExtensionAttribute()
    });
    return NewsResult(
        resultCnt: data!.length,
        data: data.map(importMap).map((data) => News.fromMap(data!)).toList());
  }

  @override
  Future<NewsResult> released() async {
    if (_userAccessToken != null) {
      if (limit != null && offset != null) {
        final data = await _channel.invokeMapMethod<String, dynamic>(
            'getPaginateReleasedSegmentNewsList', {
          'limit': limit,
          'offset': offset,
          'userAccessToken': _userAccessToken,
          'segmentMatchOnly': _segmentMatchOnly,
          'searchConditions': exportSearchConditions(),
          'sortConditions': exportSortConditions(),
          'extensionAttribute': _exportExtensionAttribute()
        });
        return NewsResult.fromMap(data!);
      }
      final data =
          await _channel.invokeListMethod<Map>('getReleasedSegmentNewsList', {
        'limit': limit,
        'userAccessToken': _userAccessToken,
        'segmentMatchOnly': _segmentMatchOnly,
        'searchConditions': exportSearchConditions(),
        'sortConditions': exportSortConditions(),
        'extensionAttribute': _exportExtensionAttribute()
      });
      return NewsResult(
          resultCnt: data!.length,
          data:
              data.map(importMap).map((data) => News.fromMap(data!)).toList());
    }
    if (limit != null && offset != null) {
      final data = await _channel
          .invokeMapMethod<String, dynamic>('getPaginateReleasedNewsList', {
        'limit': limit,
        'offset': offset,
        'searchConditions': exportSearchConditions(),
        'sortConditions': exportSortConditions(),
        'extensionAttribute': _exportExtensionAttribute()
      });
      return NewsResult.fromMap(data!);
    }
    final data = await _channel.invokeListMethod<Map>('getReleasedNewsList', {
      'limit': limit,
      'searchConditions': exportSearchConditions(),
      'sortConditions': exportSortConditions(),
      'extensionAttribute': _exportExtensionAttribute()
    });
    return NewsResult(
        resultCnt: data!.length,
        data: data.map(importMap).map((data) => News.fromMap(data!)).toList());
  }

  Map<String, dynamic> _exportExtensionAttribute() {
    return <String, dynamic>{
      'userAccessToken': _userAccessToken,
      'showFavorite': _withFavorite,
      'showFavoriteSummary': _withFavoriteSummary
    };
  }
}

mixin MethodChannelNewsClient on RKZClientPlatform
    implements MethodChannelClientInterface {
  @override
  NewsQueryPlatform newsQuery({String? userAccessToken}) {
    return MethodChannelNewsQuery(channel, userAccessToken: userAccessToken);
  }

  @override
  Future<News> getNews(String newsId, {String? newsTenantId}) async {
    final data = await channel.invokeMapMethod<String, dynamic>(
        'getNews', {'newsId': newsId, 'newsTenantId': newsTenantId});
    return News.fromMap(data!);
  }

  @override
  Future<void> readNews(String userAccessToken, String newsId,
      {String? newsTenantId}) async {
    return channel.invokeMethod('readNews', {
      'userAccessToken': userAccessToken,
      'newsId': newsId,
      'newsTenantId': newsTenantId
    });
  }

  @override
  Future<void> addNewsToFavorite(String userAccessToken, String newsId,
      {String? newsTenantId}) async {
    return channel.invokeMethod('addNewsToFavorite',
        {'userAccessToken': userAccessToken, 'newsId': newsId});
  }

  @override
  Future<void> deleteNewsFromFavorite(String userAccessToken, String newsId,
      {String? newsTenantId}) async {
    return channel.invokeMethod('deleteNewsFromFavorite',
        {'userAccessToken': userAccessToken, 'newsId': newsId});
  }
}
