import 'package:flutter/services.dart';

import '../models/stamp_history.dart';
import '../models/stamp_rally.dart';
import '../models/stamp_rally_spot.dart';
import '../platform_interface/client.dart';
import '../platform_interface/stamp_query.dart';
import '../utils/mappings.dart';
import 'method_channel_client.dart';

class MethodChannelStampRallyQuery extends StampRallyQueryPlatform
    with MethodChannelQuery {
  late MethodChannel _channel;

  MethodChannelStampRallyQuery(MethodChannel channel) {
    _channel = channel;
  }

  @override
  Future<List<StampRally>> all() async {
    final data = await _channel.invokeListMethod<Map>('getAllStampRallyList', {
      'searchConditions': exportSearchConditions(),
      'sortConditions': exportSortConditions()
    });
    return data!
        .map(importMap)
        .map((data) => StampRally.fromMap(data!))
        .toList();
  }

  @override
  Future<List<StampRally>> current() async {
    final data = await _channel.invokeListMethod<Map>('getStampRallyList', {
      'searchConditions': exportSearchConditions(),
      'sortConditions': exportSortConditions()
    });
    return data!
        .map(importMap)
        .map((data) => StampRally.fromMap(data!))
        .toList();
  }
}

class MethodChannelStampRallySpotQuery extends StampRallySpotQueryPlatform
    with MethodChannelQuery {
  late MethodChannel _channel;

  MethodChannelStampRallySpotQuery(MethodChannel channel) {
    _channel = channel;
  }

  @override
  Future<List<StampRallySpot>> find() async {
    final data = await _channel.invokeListMethod<Map>('getStampRallySpotList', {
      'searchConditions': exportSearchConditions(),
      'sortConditions': exportSortConditions()
    });
    return data!
        .map(importMap)
        .map((data) => StampRallySpot.fromMap(data!))
        .toList();
  }

  @override
  Future<List<StampRallySpot>> findByStampRallyId(String stampRallyId) async {
    final data = await _channel
        .invokeListMethod<Map>('getStampRallySpotListByStampRallyId', {
      'stampRallyId': stampRallyId,
      'searchConditions': exportSearchConditions(),
      'sortConditions': exportSortConditions()
    });
    return data!
        .map(importMap)
        .map((data) => StampRallySpot.fromMap(data!))
        .toList();
  }

  @override
  Future<List<StampRallySpot>> findBySpotId(String spotId) async {
    final data =
        await _channel.invokeListMethod<Map>('getStampRallySpotListBySpotId', {
      'spotId': spotId,
      'searchConditions': exportSearchConditions(),
      'sortConditions': exportSortConditions()
    });
    return data!
        .map(importMap)
        .map((data) => StampRallySpot.fromMap(data!))
        .toList();
  }

  @override
  Future<List<StampRallySpot>> findByBeaconId(String beaconId) async {
    final data = await _channel
        .invokeListMethod<Map>('getStampRallySpotListByBeaconId', {
      'beaconId': beaconId,
      'searchConditions': exportSearchConditions(),
      'sortConditions': exportSortConditions()
    });
    return data!
        .map(importMap)
        .map((data) => StampRallySpot.fromMap(data!))
        .toList();
  }
}

class MethodChannelStampHistoryQuery extends StampHistoryQueryPlatform
    with MethodChannelQuery {
  late MethodChannel _channel;
  String? _userAccessToken;

  MethodChannelStampHistoryQuery(
      MethodChannel channel, String userAccessToken) {
    _channel = channel;
    _userAccessToken = userAccessToken;
  }

  @override
  Future<List<StampHistory>> find() async {
    final data = await _channel.invokeListMethod<Map>('getMyStampHistoryList', {
      'userAccessToken': _userAccessToken,
      'searchConditions': exportSearchConditions(),
      'sortConditions': exportSortConditions()
    });
    return data!
        .map(importMap)
        .map((data) => StampHistory.fromMap(data!))
        .toList();
  }
}

mixin MethodChannelStampClient on RKZClientPlatform
    implements MethodChannelClientInterface {
  @override
  StampRallyQueryPlatform stampRallyQuery() {
    return MethodChannelStampRallyQuery(channel);
  }

  @override
  StampRallySpotQueryPlatform stampRallySpotQuery() {
    return MethodChannelStampRallySpotQuery(channel);
  }

  @override
  Future<void> completeStampRally(
      String userAccessToken, String stampRallyId) async {
    return channel.invokeMethod('stampComplete',
        {'userAccessToken': userAccessToken, 'stampRallyId': stampRallyId});
  }

  @override
  Future<void> addStampHistory(String userAccessToken, String stampRallyId,
      String stampRallySpotCd) async {
    return channel.invokeMethod('addMyStamp', {
      'userAccessToken': userAccessToken,
      'stampRallyId': stampRallyId,
      'stampRallySpotCd': stampRallySpotCd
    });
  }

  @override
  StampHistoryQueryPlatform stampHistoryQuery(String userAccessToken) {
    return MethodChannelStampHistoryQuery(channel, userAccessToken);
  }
}
