import 'package:flutter/services.dart';

import '../models/order.dart';
import '../models/user_detail.dart';
import '../models/user_detail_result.dart';
import '../platform_interface/client.dart';
import '../platform_interface/user_detail_query.dart';
import '../types.dart';
import '../utils/mappings.dart';
import 'method_channel_client.dart';

class MethodChannelUserDetailQuery extends UserDetailQueryPlatform
    with MethodChannelQuery {
  late MethodChannel _channel;

  String? _objectId;
  String? _userAccessToken;

  bool? _withFavorite;
  bool? _withFavoriteSummary;

  MethodChannelUserDetailQuery(MethodChannel channel, String objectId,
      {String? userAccessToken}) {
    _channel = channel;
    _objectId = objectId;
    _userAccessToken = userAccessToken;
  }

  @override
  void withFavorite({bool? byMe}) {
    _withFavorite = true;
    if (byMe != null) {
      equalTo('sys_favorite:is_favorite', exportBool(byMe)!);
    }
  }

  @override
  void withFavoriteSummary() {
    _withFavoriteSummary = true;
  }

  @override
  void orderByFavoriteUpdatedAt({Direction direction = Direction.asc}) {
    order.add(
        Order(direction: direction, columnName: 'sys_favorite:update_dte'));
  }

  @override
  void orderByFavoriteCount({Direction direction = Direction.asc}) {
    order.add(Order(
        direction: direction, columnName: 'sys_favorite_sum:favorite_count'));
  }

  Map<String, dynamic> _exportExtensionAttribute() {
    return <String, dynamic>{
      'userAccessToken': _userAccessToken,
      'showFavorite': _withFavorite,
      'showFavoriteSummary': _withFavoriteSummary
    };
  }

  @override
  Future<UserDetailResult> find() async {
    if (limit != null) {
      final data = await _channel
          .invokeMapMethod<String, dynamic>('getPaginateUserDetailList', {
        'objectId': _objectId,
        'userAccessToken': _userAccessToken ?? '',
        'limit': limit,
        'offset': offset ?? 0,
        'searchConditions': exportSearchConditions(),
        'sortConditions': exportSortConditions(),
        'extensionAttribute': _exportExtensionAttribute(),
      });
      return UserDetailResult.fromMap(data!);
    }
    final data = await _channel.invokeListMethod<Map>('getUserDetailList', {
      'objectId': _objectId,
      'userAccessToken': _userAccessToken ?? '',
      'searchConditions': exportSearchConditions(),
      'sortConditions': exportSortConditions(),
      'extensionAttribute': _exportExtensionAttribute()
    });
    return UserDetailResult(
        resultCnt: data!.length,
        data: data
            .map(importMap)
            .map((data) => UserDetail.fromMap(data!))
            .toList());
  }

  @override
  Future<UserDetail?> findById(String id) async {
    try {
      final data =
          await _channel.invokeMapMethod<String, dynamic>('getUserDetail', {
        'objectId': _objectId,
        'userAccessToken': _userAccessToken ?? '',
        'userDetailId': id,
        'extensionAttribute': _exportExtensionAttribute(),
      });
      return UserDetail.fromMap(data!);
    } on PlatformException catch (error) {
      if (error.code == RKZStatus.errorMismatch) {
        return null;
      }
      rethrow;
    }
  }

  @override
  Future<UserDetailResult> shared({List<VisibilityType>? visibility}) async {
    if (limit != null) {
      final data = await _channel
          .invokeMapMethod<String, dynamic>('getPaginateSharedUserDetailList', {
        'objectId': _objectId,
        'userAccessToken': _userAccessToken,
        'limit': limit,
        'offset': offset ?? 0,
        'visibility': visibility?.map((e) => e.toStringValue()).toList(),
        'searchConditions': exportSearchConditions(),
        'sortConditions': exportSortConditions(),
      });
      return UserDetailResult.fromMap(data!);
    }
    final data =
        await _channel.invokeListMethod<Map>('getSharedUserDetailList', {
      'objectId': _objectId,
      'userAccessToken': _userAccessToken,
      'visibility': visibility?.map((e) => e.toStringValue()).toList(),
      'searchConditions': exportSearchConditions(),
      'sortConditions': exportSortConditions(),
    });
    return UserDetailResult(
        resultCnt: data!.length,
        data: data
            .map(importMap)
            .map((data) => UserDetail.fromMap(data!))
            .toList());
  }

  @override
  Future<UserDetail?> sharedById(String id,
      {List<VisibilityType>? visibility}) async {
    try {
      final data = await _channel
          .invokeMapMethod<String, dynamic>('getSharedUserDetail', {
        'objectId': _objectId,
        'userAccessToken': _userAccessToken,
        'userDetailId': id,
        'visibility': visibility?.map((e) => e.toStringValue()).toList(),
      });
      return UserDetail.fromMap(data!);
    } on PlatformException catch (error) {
      if (error.code == RKZStatus.errorMismatch) {
        return null;
      }
      rethrow;
    }
  }

  @override
  Future<int> delete() async {
    return (await _channel.invokeMethod<int>('deleteUserDetail', {
      'objectId': _objectId,
      'userAccessToken': _userAccessToken ?? '',
      'searchConditions': exportSearchConditions(),
    }))!;
  }
}

mixin MethodChannelUserDetailClient on RKZClientPlatform
    implements MethodChannelClientInterface {
  @override
  UserDetailQueryPlatform userDetailQuery(String objectId,
      {String? userAccessToken}) {
    return MethodChannelUserDetailQuery(channel, objectId,
        userAccessToken: userAccessToken);
  }

  @override
  Future<UserDetail> addUserDetail(
      String userAccessToken, UserDetail userDetail) async {
    final data =
        await channel.invokeMapMethod<String, dynamic>('addUserDetail', {
      'userAccessToken': userAccessToken,
      'userDetail': userDetail.toMap(),
    });
    return UserDetail.fromMap(data!);
  }

  @override
  Future<UserDetail> editUserDetail(
      String userAccessToken, UserDetail userDetail) async {
    final data =
        await channel.invokeMapMethod<String, dynamic>('editUserDetail', {
      'userAccessToken': userAccessToken,
      'userDetail': userDetail.toMap(),
    });
    return UserDetail.fromMap(data!);
  }

  @override
  Future<void> addUserDetailToFavorite(
      String userAccessToken, String objectId, String id) async {
    return channel.invokeMethod('addUserDetailToFavorite', {
      'userAccessToken': userAccessToken,
      'objectId': objectId,
      'userDetailId': id,
    });
  }

  @override
  Future<void> deleteUserDetailFromFavorite(
      String userAccessToken, String objectId, String id) async {
    return channel.invokeMethod('deleteUserDetailFromFavorite', {
      'userAccessToken': userAccessToken,
      'objectId': objectId,
      'userDetailId': id,
    });
  }
}
